<?php if (!defined('PmWiki')) exit();
/* 
   publish page collection as book script, version 2.0.28
   Copyright 2003-2010 John Rankin john.rankin@affinity.co.nz
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published
   by the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This file turns a trail page into a typeset book of the contents, in a
   form suitable for printing, via the (:typeset-book:) directive.
*/

function HandleBookPublish($pagename, $auth = 'read') {
  global $HandlePublishFmt,$SpaceWikiWords,$HeadingDepth,$format;
  global $PublishFormPageFmt,$HTMLHeaderFmt,$PdfFrontMatterPattern;
  global $ExtractTrailStopsFunction,$PrintThisPageFunction,$XMLEncodingFmt;
  global $BookTrail,$PageAnchors;  $BookTrail = true;
  SDV($ExtractTrailStopsFunction, 'ExtractTrailStops');
  SDV($PrintThisPageFunction, 'PrintThisPage');
  $Tp    = 'part';  $Ta = 'appendix';
  $Tu    = $Tp.' class="unnumber"';
  $Tbook = array('chapter',
     'section','subsection','subsubsection','paragraph','subparagraph');
  $Chapter  = 'E:blockquote:';
  $TbookPTV = array('epigraph'=>'E', 'dedication'=>'K', 'about'=>'K', 'isbn'=>'I', 
                    'graphics'=>'V', 'year'=>'I');
  $TbookDir = array('description' => 'publisher', 'keywords' => 'keywords');
  $TbookCCL = array('rights' => array('^(none|cc|gfdl)' => 'CopyrightClassFmt',
                                      '^(cc\s+by(-nc)?(-(?:sa|nd))?)$' => 'CopyrightStyleFmt',
                                      '"([^"]+)"$' => 'CopyrightStyleFmt'));
  $TbookOpt = array('typeset-book' => 'SetPdfMetaTags', 'cfloat' => 'SetCfloatName',
                    'blurb'        => 'SetBookBlurb');
  $TitleFmt = ($SpaceWikiWords) ? '$Titlespaced' : '$Title';
  $Unnumber = array('preface', 'introduction', 'acknowledgments', 'colophon',
               'prologue', 'epilogue', 'foreword', 'afterword', 'unnumber');

  $trailname = $pagename;
  $trailpage = RetrieveAuthPage($pagename, $auth, true, READPAGE_CURRENT);
  if (!$trailpage) Abort('?cannot read $pagename');
  PCache($pagename,$trailpage);
  if ($format=='pdf') {
      $Chapter = 'E:aphorism:';
      Markup('tbepi','block','/^E:([^:]+):(.*?)(?:\s*&mdash;\s*(.*?)\s*("[^"]+")?)?$/e',
        "'<:block><tbook:$1>'.PSS('$2').
  ('$3' ? '<tbook:caption'.('$4' ? ' caption='.PSS('$4') : '').'>'.PSS('$3').'</tbook:caption>' : '').
        '</tbook:$1>'");
      Markup('tblogo','block','/^V:([^:]+):(.*?)$/','<:block>$2');
      Markup('tbdedn','<^*','/^K:([^:]+):(.*?)$/e',
  "'<:block><tbook:$1>'.str_replace(':+','<tbook:newline vspace=\"1ex\"/>',PSS('$2')).'</tbook:$1>'");
      Markup('tbyear','block','/^I:([^:]+):(.*?)\s*("[^"]+")?$/e',
          "'<:block><tbook:$1'.('$3' ? ' caption='.PSS('$3') : '').'>$2</tbook:$1>'");
      preg_match_all('/\\(:(\\w[-\\w]+):?(?!\\))(.*?):\\)/',$trailpage['text'],$dir,PREG_SET_ORDER);
      foreach ($dir as $k) $meta[$k[1]] = trim($k[2]);
      foreach ($TbookPTV as $k => $v)
          if (isset($meta[$k])) array_unshift($HTMLHeaderFmt, "markup:$v:$k:$meta[$k]");
      foreach ($TbookDir as $k => $v)
          if (isset($meta[$k])) array_unshift($HTMLHeaderFmt,
             "\n<$v>".htmlentities($meta[$k],ENT_NOQUOTES,$XMLEncodingFmt)."</$v>");
      foreach ($TbookCCL as $k => $va)
          if (isset($meta[$k]))
              foreach ($va as $mat => $glo)
                  if (preg_match("/$mat/i",$meta[$k],$gv)) $GLOBALS[$glo] = $gv[1];
      foreach ($TbookOpt as $k => $v) if (isset($meta[$k])) $v($meta[$k]);
      SDVA($HTMLHeaderFmt,
          PublishFormDefaults($trailname,$PublishFormPageFmt['book']));
  } else
      Markup('tbchap','block','/^E:([^:]+):(.*?)(?:\s*&mdash;\s*(.*?)\s*("[^"]+")?)?$/e',
        "'<:block><$1>'.PSS('$2').
        ('$3' ? ' &mdash; <cite>'.PSS('$3').'</cite>' : '').
        ('$4' ? ', <strong>'.PSS('$4').'</strong>'    : '').'</$1>'");
  $FM = FrontMatter('default');
  foreach ($PdfFrontMatterPattern as $k => $v) SDV($GLOBALS[$v], $FM[$k]);
  $trailpage['text'] = str_replace("\\\\\n",'[[&lt;&lt;]]',$trailpage['text']);
  $aphorisms = ExtractChapterAphorisms($trailname,$trailpage['text']);
  $dpl = $ExtractTrailStopsFunction($trailname,$trailpage['text']);
  foreach ($dpl as $k => $v) $$k = $v;

    $counter = array(); $predepth= 0; $appendix = false; $part = false;
    for($i=0;$i<count($pages);$i++) if ($depth[$i]) {
      @$counter[$depth[$i]-1]++;
      $num = $counter[0];
      for($j=$depth[$i];$j<$predepth;$j++) $counter[$j] = 0;
      $size = 0;
      for($j=1;$j<$depth[$i];$j++) { $num .= "." . $counter[$j]; $size++; }
      if ($size>3) $size=3;
      
      $anchor = "wikipublisher.$num";
      $num = "[[#$anchor]]";

      if ($format=='pdf') {
          $olink = $links[$i];
          $tbtag = '<'.$Tbook[$depth[$i]-1];
          if ($depth[$i]==1) {
            $linktxt = TextOfLink($trailname, $links[$i]);
            $savetxt = $linktxt;
            if (preg_match('/^[Aa]ppendix[:\s]\s*(.*)$/',$linktxt,$m)) {
              if (preg_match('/^([Uu]n)?[Pp]art:([^:]*):\s*(.*)$/',$partname.$m[1],$p)) {
                  $m[1] = $p[3];
                  $pun  = $p[1] ? $Tu : $Tp;
                  $tbtag= "<$pun>".MarkupToHTML($pagename,"H:$p[2]").$tbtag;
                  $tbtag= ($part ? "</$Tp>" : '').($appendix ? '' : "<$Ta>").$tbtag;
                  $part = true;
              } else{
                  $tbtag= ($appendix ? '' : ($part ? "</$Tp>" : '')."<$Ta>").$tbtag;
                  $part = ($appendix ? $part : false);
              }
              $olink    = str_replace($m[0],$m[1],$links[$i]);
              $savetxt  = $m[1];
              $appendix = true;
            } elseif (preg_match('/^((?:[Uu]n)?[Nn]umber|[Pp]reface):\s*(.*)$/',$linktxt,$m)) {
              $olink    = str_replace($m[0],$m[2],$links[$i]);
              $linktxt  = $m[1];
              $savetxt  = $m[2];
              if ($part) { $part = false;  $tbtag = "</$Tp>" . $tbtag; }
            } elseif (preg_match('/^([Aa]fterword)(:\s*(.*))?$/',$linktxt,$m)) {
              if ($m[2]) { $olink = str_replace($m[0],$m[3],$links[$i]); $savetxt  = $m[3]; }
              $linktxt  = $m[1];
              if ($part) { $part = false;  $tbtag = "</$Tp>" . $tbtag; }
            } elseif (preg_match('/^([Uu]n)?[Pp]art:([^:]*):\s*(.*)$/',MarkupEscape($partname.$linktxt),$m)) {
              $m = MarkupRestore($m);
              $olink    = str_replace($m[0],$m[3],$links[$i]);
              $savetxt  = $m[3];
              $pun      = $m[1] ? $Tu : $Tp;
              $tbtag    = ($part ? "</$Tp>" : '')."<$pun>".MarkupToHTML($pagename,"H:$m[2]").$tbtag;
              $part     = true;
            }
            $partname = '';
            if (preg_match('/"([^"]+)"$/',$savetxt,$hdg))
                $olink = str_replace($hdg[0],'',$olink).$hdg[0];
            $tbtag .= ChapterKind(strtolower($linktxt), $Unnumber);
          }
          $tbtag .= '>';
          for($j=$depth[$i];$j<=$predepth;$j++) 
            $tbtag = '</'.$Tbook[$j-1].'>'.$tbtag;
          $PublishList[] = $tbtag;
          $PublishList[] = "function:PrintThisMarkup H:$num$olink";
      } else {
          if ($depth[$i]==1) {
            $linktxt = MarkupEscape($partname.TextOfLink($trailname,$links[$i]));
            if (preg_match('/^([Aa]ppendix[:\s])?([Uu]n)?[Pp]art:([^:]*):\s*(.*)$/',$linktxt,$m)) {
               $m = MarkupRestore($m);
               $links[$i] = str_replace($m[0],$m[1].$m[4],$links[$i]);
               $PublishList[] = "markup:!%block class='pagehead0'%".$m[3];
               $partname = '';
            }
          }
          $s = str_repeat('-',$size);
          $h = "!%block class='pagehead$depth[$i]'%";
          if ($size==0) $PublishList[] = "markup:$h$num$links[$i]";
          else $PublishList[] = "markup:$h"."[$s$num$links[$i]$s]";
      }      
      $predepth = $depth[$i];

      if ($pages[$i]!="textonly") {
          $PageAnchors[$pages[$i]]  = $anchor;
          $HeadingDepth[$pages[$i]] = $Tbook[$depth[$i]];
          if (isset($aphorisms[$pages[$i]])) 
              $PublishList[] = "markup:$Chapter".$aphorisms[$pages[$i]];
          if (PageExists($pages[$i])) 
	       	  $PublishList[] = "function:$PrintThisPageFunction $pages[$i]";
          elseif ($depth[$i]>1) {
              $replname = TextOfLink($trailname,$links[$i]);
              $PublishList[] = "function:PrintThisText " . 
    ((MakeLink($trailname,"[[$replname]]",NULL,NULL,'$FullName')==$pages[$i]) ?
                $links[$i] : "[[".$pages[$i]."]] ($replname)");
          }
      }
    } else $partname = $links[$i];

    if ($format=='pdf') {
        $tbtag = '';
        for($j=1;$j<=$predepth;$j++) 
            $tbtag = '</'.$Tbook[$j-1].'>'.$tbtag;
        if ($part)     $tbtag.= "</$Tp>";
        if ($appendix) $tbtag.= "</$Ta>";
        $PublishList[]  = $tbtag;
        $PublishList[0] = 
          str_replace('<chapter','<chapter id="wikipublisher.top"',$PublishList[0]);
    } else array_unshift($PublishList,'markup:[[#wikipublisher.top]]');

  $GLOBALS['PublishList'] = $PublishList;
  PrintFmt($pagename,$HandlePublishFmt);
}

function ExtractChapterAphorisms($trailname,$trailtext) {
  global $SuffixPattern; $aphorisms = array();
  foreach(explode("\n",$trailtext) as $x) {
      if (preg_match("/^:\\s*([[:upper:]\\[].*)/",$x,$match)) {
          $match[1] = ReformatTrailStop(preg_replace("/\\{(\\$.*?)\\}/e",
                "FmtPageName('$1',\$trailname)",$match[1]));
          if (preg_match("/^(\\[\\[([^|:]*?)(?:\\|([^\\[\\]]*?(?:\\[.*?\\].*?)?))?\\]\\]($SuffixPattern))\\s*:(.*)/",
          $match[1],$m)) {
              $aphorisms[MakePageName($trailname,$m[2])] = $m[5];
          }
      }
  }
  return $aphorisms;
}

function ChapterKind($kind, $att) {
  global $ChapterKindFmt;
  for ($j=0;$j<count($att);$j++)
    if ($kind==(isset($ChapterKindFmt[$att[$j]]) ? strtolower($ChapterKindFmt[$att[$j]]) : $att[$j]))
        return " kind='$att[$j]'";
  return '';
}

function SetBookBlurb($opt) {
  global  $NamePattern;
  if (preg_match("/^$NamePattern/",$opt,$m)) XLSDV('en', array('PublisherBlurb' => $m[0]));
  XLSDV('en', array('PraiseFor' => (preg_match('/"([^"]+)"$/',$opt,$m)) ? $m[1] : 'Praise for'));
  SetTmplDisplay('PageBlurbFmt', 1);          
}

?>